/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls/internal/platform.h
 *
 * Purpose:     Platform discrimination for the recls API.
 *
 * Created:     15th August 2003
 * Updated:     24th December 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright (c) 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef RECLS_INCL_RECLS_INTERNAL_H_PLATFORM
#define RECLS_INCL_RECLS_INTERNAL_H_PLATFORM

/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_RECLS_INTERNAL_H_PLATFORM_MAJOR      3
# define RECLS_VER_RECLS_INTERNAL_H_PLATFORM_MINOR      0
# define RECLS_VER_RECLS_INTERNAL_H_PLATFORM_REVISION   1
# define RECLS_VER_RECLS_INTERNAL_H_PLATFORM_EDIT       19
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/** \file recls/internal/platform.h Platform detection for the \ref group_recls  API */

/* /////////////////////////////////////////////////////////////////////////////
 * Platform recognition
 *
 * Define the symbol RECLS_OVERRIDE_PLATFORM to provide your own platform
 * discrimination
 */

#ifndef RECLS_OVERRIDE_PLATFORM
# if defined(WIN32) || \
     defined(_WIN32)
#  define RECLS_PLATFORM_IS_WIN32
# elif defined(unix) || \
       defined(UNIX) || \
       defined(__unix) || \
       defined(__unix__) || \
       ( defined(__xlC__) && \
         defined(_POWER) && \
         defined(_AIX))
#  define RECLS_PLATFORM_IS_UNIX
# else
#  error Platform not (yet) recognised
# endif /* platform */
#endif /* !RECLS_OVERRIDE_PLATFORM */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#if defined(RECLS_PLATFORM_IS_WIN32)
# if !defined(RECLS_PURE_API)
#  include <windows.h>
# endif /* !RECLS_PURE_API */
#elif defined(RECLS_PLATFORM_IS_UNIX)
# include <unistd.h>
#elif defined(RECLS_OVERRIDE_PLATFORM)
  /* Assume that the appropriate inclusions are made */
#else
# error Platform not (yet) recognised
#endif /* platform */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Constants and definitions
 */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !RECLS_INCL_RECLS_INTERNAL_H_PLATFORM */

/* ////////////////////////////////////////////////////////////////////////// */
