/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls/stl/recls.hpp
 *
 * Purpose:     recls STL mapping.
 *
 * Created:     10th September 2003
 * Updated:     27th December 2005
 *
 * Home:        http://recls.org/
 *
 * Copyright (c) 2003-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/** \file recls/stl/recls.hpp The root header for the \ref group_recls_stl "recls STL mapping" */

#ifndef RECLS_INCL_RECLS_STL_HPP_RECLS
#define RECLS_INCL_RECLS_STL_HPP_RECLS

/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_RECLS_STL_HPP_RECLS_MAJOR    4
# define RECLS_VER_RECLS_STL_HPP_RECLS_MINOR    0
# define RECLS_VER_RECLS_STL_HPP_RECLS_REVISION 2
# define RECLS_VER_RECLS_STL_HPP_RECLS_EDIT     18
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <recls/recls.h>

#if RECLS_VER < RECLS_VER_1_9_1
# error The recls STL mapping requires version 1.9.1 or later of the recls C API
#endif /* RECLS_VER < 1.9.1 */

// FTP?
#if defined(RECLS_PLATFORM_IS_WIN32)
# if !defined(RECLS_INCL_H_RECLS_FTP)
#  include <recls/ftp.h>
# endif /* !RECLS_INCL_H_RECLS_FTP */
#endif /* RECLS_PLATFORM_IS_WIN32 */

#include <stlsoft/stlsoft.h>    /* If the compiler cannot find this, you are not using STLSoft 1.9.1 or later, as required. */
#if !defined(_STLSOFT_VER_1_9_1)
# error STLSoft version 1.9.1 or greater required. (www.stlsoft.org/downloads.html)
#endif /* STLSoft version */

#ifdef RECLS_STL_STRING_HEADER
# include RECLS_STL_STRING_HEADER
#else
# include <string>
#endif /* RECLS_STL_STRING_HEADER */

#ifdef RECLS_STL_METHOD_PROPERTY_SUPPORT
# undef RECLS_STL_METHOD_PROPERTY_SUPPORT
#endif /* RECLS_STL_METHOD_PROPERTY_SUPPORT */

#if defined(RECLS_STL_FORCE_METHOD_PROPERTY_SUPPORT)
# define RECLS_STL_METHOD_PROPERTY_SUPPORT
#elif defined(RECLS_STL_NO_METHOD_PROPERTY_SUPPORT)
 /* Do not define RECLS_STL_METHOD_PROPERTY_SUPPORT */
#else /* ? RECLS_STL_???_METHOD_PROPERTY_SUPPORT */
# if defined(STLSOFT_COMPILER_IS_BORLAND)
 /* Do not define RECLS_STL_METHOD_PROPERTY_SUPPORT */
# elif defined(STLSOFT_COMPILER_IS_DMC)
#  define RECLS_STL_METHOD_PROPERTY_SUPPORT
# elif defined(STLSOFT_COMPILER_IS_GCC) && \
       (   __GNUC__ >= 4 || \
           (   __GNUC__ == 3 && \
               __GNUC_MINOR__ >= 3))
#  define RECLS_STL_METHOD_PROPERTY_SUPPORT
# elif defined(STLSOFT_COMPILER_IS_INTEL)
#  define RECLS_STL_METHOD_PROPERTY_SUPPORT
# elif defined(STLSOFT_COMPILER_IS_MSVC)
#  if _MSC_VER >= 1310
#   define RECLS_STL_METHOD_PROPERTY_SUPPORT
#  endif /* _MSC_VER */
# elif defined(STLSOFT_COMPILER_IS_MWERKS)
#  define RECLS_STL_METHOD_PROPERTY_SUPPORT
# endif /* compiler */
#endif /* RECLS_STL_???_METHOD_PROPERTY_SUPPORT */

#ifdef RECLS_STL_METHOD_PROPERTY_SUPPORT
# include <stlsoft/method_properties.hpp>
#endif /* RECLS_STL_METHOD_PROPERTY_SUPPORT */

/* /////////////////////////////////////////////////////////////////////////////
 * Property Macros - borrowed from STLSoft's XMLSTL sub-project
 */

#ifdef RECLS_STL_METHOD_PROPERTY_SUPPORT
# define RECLS_STL_OPT_METHOD_PROPERTY_GET(V, R, C, GM, P)                  STLSOFT_METHOD_PROPERTY_GET(V, R, C, GM, P)
# define RECLS_STL_OPT_METHOD_PROPERTY_GET_PROP(R, C, GM, P)                STLSOFT_METHOD_PROPERTY_GET_EXTERNAL_PROP(R, C, GM, P)
# define RECLS_STL_OPT_METHOD_PROPERTY_GETSET(V, RG, RS, C, GM, SM, P)      STLSOFT_METHOD_PROPERTY_GETSET_EXTERNAL(RG, RS, C, GM, SM, P)
# define RECLS_STL_OPT_METHOD_PROPERTY_DEFINE_OFFSET(C, P)                  STLSOFT_METHOD_PROPERTY_DEFINE_OFFSET(C, P)
# define RECLS_STL_OPT_METHOD_PROPERTY_GETSET_PROP(RG, RS, C, GM, SM, P)    STLSOFT_METHOD_PROPERTY_GETSET_EXTERNAL_PROP(RG, RS, C, GM, SM, P)
#else /* ? RECLS_STL_METHOD_PROPERTY_SUPPORT */
# define RECLS_STL_OPT_METHOD_PROPERTY_GET(V, R, C, GM, P)
# define RECLS_STL_OPT_METHOD_PROPERTY_GET_PROP(R, C, GM, P)
# define RECLS_STL_OPT_METHOD_PROPERTY_GETSET(V, RG, RS, C, GM, SM, P)
# define RECLS_STL_OPT_METHOD_PROPERTY_DEFINE_OFFSET(C, P)
# define RECLS_STL_OPT_METHOD_PROPERTY_GETSET_PROP(RG, RS, C, GM, SM, P)
#endif /* RECLS_STL_METHOD_PROPERTY_SUPPORT */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{

namespace stl
{
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs
 */

#ifdef RECLS_DOCUMENTATION_SKIP_SECTION

/// \brief string_t The string type to be used in the \c reclstl (\c recls::stl) namespace
///
/// \ingroup group_recls_stl
/// 
/// \note By default \c string_t is defined to be <code>std::string</code>, but this can be
/// overridden by specifying \c RECLS_STL_STRING_HEADER, e.g. <code>"mystring.h"</code>,
/// <b>and</b> also defining \c RECLS_STL_STRING_T, e.g. <code>"my_string"</code> which will
/// be typedef'd to \c string_t
typedef std::string             string_t;

#else /* ? RECLS_DOCUMENTATION_SKIP_SECTION */

# ifdef RECLS_STL_STRING_HEADER
#  ifndef RECLS_STL_STRING_T
#   error RECLS_STL_STRING_T must be defined when RECLS_STL_STRING_HEADER is defined
#  endif /* !RECLS_STRING_T */
typedef RECLS_STL_STRING_T          string_t;
# else /* ? RECLS_STRING_HEADER */
typedef stlsoft_ns_qual_std(string) string_t;
# endif /* RECLS_STRING_HEADER */
#endif /* RECLS_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// \brief The exception type thrown by the Open-RJ - STL mapping
class recls_exception
    : public std::exception
{
/// \name Types
/// @{
public:
    typedef std::exception      parent_class_type;
    typedef recls_exception  class_type;
/// @}

/// \name Construction
/// @{
public:
    /// Constructs an exception from the given error code and error structure
    explicit recls_exception(recls_rc_t rc)
        : m_rc(rc)
        , m_message(create_message_(rc))
    {}
#ifdef STLSOFT_COMPILER_IS_GCC
    virtual ~recls_exception() throw()
    {}
#endif /* STLSOFT_COMPILER_IS_GCC */
/// @}

/// \name Accessors
/// @{
public:
    /// \brief The Open-RJ return code
    recls_rc_t rc() const
    {
        return m_rc;
    }

    /// \brief A string form of the exception error message
    virtual char const *what() const throw()
    {
        return m_message.c_str();
    }
/// @}

/// \name Implementation
/// @{
private:
    static string_t create_message_(recls_rc_t rc)
    {
        string_t    msg(recls::Recls_GetErrorString(rc, NULL, 0), '~');

        recls::Recls_GetErrorString(rc, &msg[0], msg.size());

        return msg;
    }
/// @}

/// \name Members
/// @{
private:
    const recls_rc_t    m_rc;
    const string_t      m_message;
/// @}

// Not to be implemented
private:
    class_type &operator =(class_type const &);
};

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace stl */
} /* namespace recls */
namespace reclstl = ::recls::stl;
#endif /* !RECLS_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !RECLS_INCL_RECLS_STL_HPP_RECLS */

/* ////////////////////////////////////////////////////////////////////////// */
