////////////////////////////////////////////////////////////////////////////////
// FileSearch.cpp : Implementation of FileSearch
//
// Updated: 3rd December 2005

#include "stdafx.h"
#include "recls.com.h"
#include "FileSearch.h"
#include "SearchCollection.h"

////////////////////////////////////////////////////////////////////////////////
// IFileSearch

STDMETHODIMP FileSearch::Search(BSTR searchRoot, BSTR pattern, long flags, IUnknown **ppenum)
{
	HRESULT	hr;

	if(NULL == ppenum)
	{
		hr = E_POINTER;
	}
	else
	{
		*ppenum = NULL;

        hrecls_t    hSrch;
		recls_rc_t	rc	=	call_Search(searchRoot
									,	pattern
									,	flags
									,	NULL
									,	&hSrch);

		if( RECLS_FAILED(rc) && 
			rc != RECLS_RC_NO_MORE_DATA)
		{
			set_error_info_from_recls_rc_t(rc);

			hr = E_FAIL;
		}
		else
		{
			typedef	CComObject<SearchCollection>	Search_t;

			Search_t	*pSrch = new Search_t();

			if(NULL == pSrch)
			{
				Recls_SearchClose(hSrch);

				hr = E_OUTOFMEMORY;
			}
			else
			{
				pSrch->AddRef();

				SearchInfo	info;

				if(rc == RECLS_RC_NO_MORE_DATA)
				{
					info.bEmpty		=	true;
				}
				else
				{
					info.hSrch		=	hSrch;
					info.flags		=	flags;
					info.pattern	=	pattern;
					info.searchRoot	=	searchRoot;
				}

				pSrch->SetVoid(&info);
				ATLASSERT(NULL == info.hSrch);

				*ppenum = pSrch;

				hr = S_OK;
			}
		}
	}

	return hr;
}

// IFileSearch2
STDMETHODIMP FileSearch::SearchProgress(/*[in]*/ BSTR searchRoot, /*[in]*/ BSTR pattern, /*[in]*/ long flags, /*[in]*/ LPUNKNOWN punkProgress, /*[out, retval]*/ IUnknown **ppenum)
{
	HRESULT	hr;

	if(NULL == ppenum)
	{
		hr = E_POINTER;
	}
	else
	{
		*ppenum = NULL;

        hrecls_t    hSrch;
		recls_rc_t	rc	=	call_Search(searchRoot
									,	pattern
									,	flags
									,	punkProgress
									,	&hSrch);

		if( RECLS_FAILED(rc) && 
			rc != RECLS_RC_NO_MORE_DATA)
		{
			set_error_info_from_recls_rc_t(rc);

			hr = E_FAIL;
		}
		else
		{
			typedef	CComObject<SearchCollection>	Search_t;

			Search_t	*pSrch = new Search_t();

			if(NULL == pSrch)
			{
				Recls_SearchClose(hSrch);

				hr = E_OUTOFMEMORY;
			}
			else
			{
				pSrch->AddRef();

				pSrch->SetProgress(punkProgress);

				SearchInfo	info;

				if(rc == RECLS_RC_NO_MORE_DATA)
				{
					info.bEmpty		=	true;
				}
				else
				{
					info.hSrch		=	hSrch;
					info.flags		=	flags;
					info.pattern	=	pattern;
					info.searchRoot	=	searchRoot;
				}

				pSrch->SetVoid(&info);
				ATLASSERT(NULL == info.hSrch);

				*ppenum = pSrch;

				hr = S_OK;
			}
		}
	}

	return hr;
}

STDMETHODIMP FileSearch::get_Roots(/* [out, retval] */ LPUNKNOWN *pVal)
{
	HRESULT	hr;

	if(NULL == pVal)
	{
		hr = E_POINTER;
	}
	else
	{
		*pVal = NULL;

		typedef	CComObject< CComEnum<IEnumString, &IID_IEnumString, LPOLESTR, _Copy<LPOLESTR> > >	EnumRoot;

		recls_root_t	roots[26];
		size_t			cRoots	=	Recls_GetRoots(roots, stlsoft_num_elements(roots));
		EnumRoot		*pe		=	new EnumRoot;

		if(NULL == pe)
		{
			hr = E_OUTOFMEMORY;
		}
		else
		{
			pe->AddRef();

			OLECHAR		ole_roots[26][4];
			LPOLESTR	root_ptrs[26];

			stlsoft_ns_qual_std(copy)(&ole_roots[0], &ole_roots[stlsoft_num_elements(ole_roots)], &root_ptrs[0]);

			for(size_t i = 0; i < cRoots; ++i)
			{
#ifdef RECLS_CHAR_TYPE_IS_WCHAR
				lstrcpyW(root_ptrs[i], roots[i].name);
#else /* ? RECLS_CHAR_TYPE_IS_WCHAR */
				::MultiByteToWideChar(0, 0, roots[i].name, 4, ole_roots[i], 4);
#endif /* RECLS_CHAR_TYPE_IS_WCHAR */
			}

			hr = pe->Init(&root_ptrs[0], &root_ptrs[cRoots], NULL, AtlFlagCopy);

			if(SUCCEEDED(hr))
			{
				hr = pe->QueryInterface(IID_IUnknown, reinterpret_cast<void**>(pVal));
			}

			pe->Release();
		}
	}

	return hr;
}

STDMETHODIMP FileSearch::get_PathNameSeparator(BSTR *pVal)
{
	HRESULT	hr;

	if(NULL == pVal)
	{
		hr = E_POINTER;
	}
	else
	{
		USES_CONVERSION;

		*pVal = ::SysAllocString(T2COLE(Recls_GetPathNameSeparator()));

		hr = (NULL == *pVal) ? E_OUTOFMEMORY : S_OK;
	}

	return hr;
}

STDMETHODIMP FileSearch::get_PathSeparator(BSTR *pVal)
{
	HRESULT	hr;

	if(NULL == pVal)
	{
		hr = E_POINTER;
	}
	else
	{
		USES_CONVERSION;

		*pVal = ::SysAllocString(T2COLE(Recls_GetPathSeparator()));

		hr = (NULL == *pVal) ? E_OUTOFMEMORY : S_OK;
	}

	return hr;
}

STDMETHODIMP FileSearch::get_WildcardsAll(BSTR *pVal)
{
	HRESULT	hr;

	if(NULL == pVal)
	{
		hr = E_POINTER;
	}
	else
	{
		USES_CONVERSION;

		*pVal = ::SysAllocString(T2COLE(Recls_GetWildcardsAll()));

		hr = (NULL == *pVal) ? E_OUTOFMEMORY : S_OK;
	}

	return hr;
}

////////////////////////////////////////////////////////////////////////////////
// IFtpSearch

STDMETHODIMP FileSearch::Search(/*[in]*/ BSTR host, /*[in]*/ BSTR username, /*[in]*/ BSTR password, /*[in]*/ BSTR searchRoot, /*[in]*/ BSTR pattern, /*[in]*/ long flags, /*[out, retval]*/ IUnknown **ppenum)
{
	HRESULT	hr;

	if(	NULL != username &&
		'\0' == *username)
	{
		username = NULL;
	}

	if(	NULL != password &&
		'\0' == *password)
	{
		password = NULL;
	}

	if(NULL == ppenum)
	{
		hr = E_POINTER;
	}
	else
	{
		*ppenum = NULL;

#if 1
        hrecls_t    hSrch;

#ifdef RECLS_CHAR_TYPE_IS_CHAR
		recls_rc_t	rc	=	Recls_SearchFtp(winstl::w2a(host), winstl::w2a(username), winstl::w2a(password), winstl::w2a(searchRoot), winstl::w2a(pattern), static_cast<recls_uint32_t>(flags), &hSrch);
#elif defined(RECLS_CHAR_TYPE_IS_WCHAR)
		recls_rc_t	rc	=	Recls_SearchFtp(host, username, password, searchRoot, pattern, static_cast<recls_uint32_t>(flags), &hSrch);
#endif /* RECLS_CHAR_TYPE_IS_WCHAR */

		if( RECLS_FAILED(rc) && 
			rc != RECLS_RC_NO_MORE_DATA)
		{
			set_error_info_from_recls_rc_t(rc);

			hr = E_FAIL;
		}
		else
		{
			typedef	CComObject<SearchCollection>	Search_t;

			Search_t	*pSrch = new Search_t();

			if(NULL == pSrch)
			{
				Recls_SearchClose(hSrch);

				hr = E_OUTOFMEMORY;
			}
			else
			{
				pSrch->AddRef();

				SearchInfo	info;

				if(rc == RECLS_RC_NO_MORE_DATA)
				{
					info.bEmpty		=	true;
				}
				else
				{
					info.hSrch		=	hSrch;
					info.flags		=	flags;
					info.pattern	=	pattern;
					info.searchRoot	=	searchRoot;
				}

				pSrch->SetVoid(&info);
				ATLASSERT(NULL == info.hSrch);

				*ppenum = pSrch;

				hr = S_OK;
			}
		}
#else
		hr = E_NOTIMPL;
#endif /* 0 */
	}

	return hr;
}
