////////////////////////////////////////////////////////////////////////////////
// recls.com.cpp : Implementation of DLL Exports.
//
// Updated: 14th December 2005


// Note: Proxy/Stub Information
//      To build a separate proxy/stub DLL, 
//      run nmake -f recls.comps.mk in the project directory.

#include "stdafx.h"
#include "resource.h"
#include <initguid.h>
#include "recls.com.h"

#include "recls.com_i.c"
#include "FileSearch.h"
#include "FileEntry.h"


ReclsModule _Module;

BEGIN_OBJECT_MAP(ObjectMap)
OBJECT_ENTRY(CLSID_FileSearch, FileSearch)
OBJECT_ENTRY(CLSID_FileEntry, FileEntry)
END_OBJECT_MAP()

////////////////////////////////////////////////////////////////////////////////

namespace
{
	long	s_currentNumberOfObjects	=	0;
	long	s_totalNumberOfObjects		=	0;
}

void ReclsModule::OnObjectConstruct()
{
	::InterlockedIncrement(&s_currentNumberOfObjects);
	::InterlockedIncrement(&s_totalNumberOfObjects);
}

void ReclsModule::OnObjectDestroy()
{
	::InterlockedDecrement(&s_currentNumberOfObjects);
}

long ReclsModule::CurrentNumberOfObjects() const
{
	return s_currentNumberOfObjects;
}

long ReclsModule::TotalNumberOfObjects() const
{
	return s_totalNumberOfObjects;
}

////////////////////////////////////////////////////////////////////////////////
// DLL Entry Point

extern "C"
BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID /*lpReserved*/)
{
    if (dwReason == DLL_PROCESS_ATTACH)
    {
        _Module.Init(ObjectMap, hInstance, &LIBID_RECLS_COMLib);
        DisableThreadLibraryCalls(hInstance);
    }
    else if (dwReason == DLL_PROCESS_DETACH)
	{
		char	sz[201];

		::wsprintfA(&sz[0]
				,	"recls.COM: Shutdown: current=%ld; total=%ld\n"
				,	_Module.CurrentNumberOfObjects()
				,	_Module.TotalNumberOfObjects());

		::OutputDebugStringA(&sz[0]);

        _Module.Term();
	}
    return TRUE;    // ok
}

////////////////////////////////////////////////////////////////////////////////
// Used to determine whether the DLL can be unloaded by OLE

STDAPI DllCanUnloadNow(void)
{
    return (_Module.GetLockCount() == 0) ? S_OK : S_FALSE;
}

////////////////////////////////////////////////////////////////////////////////
// Returns a class factory to create an object of the requested type

STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID *ppv)
{
    return _Module.GetClassObject(rclsid, riid, ppv);
}

////////////////////////////////////////////////////////////////////////////////
// DllRegisterServer - Adds entries to the system registry

STDAPI DllRegisterServer(void)
{
    // registers object, typelib and all interfaces in typelib
    return _Module.RegisterServer(TRUE);
}

////////////////////////////////////////////////////////////////////////////////
// DllUnregisterServer - Removes entries from the system registry

STDAPI DllUnregisterServer(void)
{
    return _Module.UnregisterServer(TRUE);
}

////////////////////////////////////////////////////////////////////////////////
#if 0

USER32:

	CharNextW

KERNEL32:

	FindFirstFileW
	FindNextFileW
	GetFileAttributesW
	GetFullPathNameW
	GetModuleFileNameW
	GetModuleHandleW
	GetShortPathNameW
	LoadLibraryW
	lstrcatW
	lstrcmpW
	lstrcpyW
	lstrlenW

#endif /* 0 */
////////////////////////////////////////////////////////////////////////////////

static int RECLS_CALLCONV_DEFAULT progress_fn(	recls_char_t const          *dir
											,   size_t                      dirLen
											,   recls_process_fn_param_t    param
											,   void                        *reserved0
											,   recls_uint32_t              reserved1)
{
	_ASSERT(NULL != param);

	IFileSearchProgress	*fsp	=	(IFileSearchProgress*)param;
	VARIANT_BOOL		bCancel	=	VARIANT_FALSE;
	CComBSTR			bstrDir(dirLen, dir);
	HRESULT				hr		=	fsp->OnDirectoryScan(bstrDir, &bCancel);

	return (FAILED(hr) || VARIANT_FALSE != bCancel) ? 0 : 1;
}

recls_rc_t call_Search(LPCOLESTR searchRoot, LPCOLESTR pattern, long flags, LPUNKNOWN punkProgress, hrecls_t *phSrch)
{
	recls_rc_t			rc;
	IFileSearchProgress	*fsp;

	if( NULL == punkProgress ||
		FAILED(punkProgress->QueryInterface(IID_IFileSearchProgress, reinterpret_cast<void**>(&fsp))))
	{
		fsp = NULL;
	}

	if(NULL != fsp)
	{
#ifdef RECLS_CHAR_TYPE_IS_CHAR
		rc = Recls_SearchFeedback(	winstl::w2a(searchRoot)
								,	winstl::w2a(pattern)
#elif defined(RECLS_CHAR_TYPE_IS_WCHAR)
		rc = Recls_SearchFeedback(	searchRoot
								,	pattern
#endif /* RECLS_CHAR_TYPE_IS_WCHAR */
								,	static_cast<recls_uint32_t>(flags)
								,	progress_fn
								,	fsp
								,	phSrch);
	}
	else
	{
#ifdef RECLS_CHAR_TYPE_IS_CHAR
		rc = Recls_Search(	winstl::w2a(searchRoot)
						,	winstl::w2a(pattern)
#elif defined(RECLS_CHAR_TYPE_IS_WCHAR)
		rc = Recls_Search(	searchRoot
						,	pattern
#endif /* RECLS_CHAR_TYPE_IS_WCHAR */
						,	static_cast<recls_uint32_t>(flags)
						,	phSrch);
	}

	if(NULL != fsp)
	{
		fsp->Release();
	}

	return rc;
}

////////////////////////////////////////////////////////////////////////////////
