/* /////////////////////////////////////////////////////////////////////////////
 * File:        test_SearchProcess_C.c
 *
 * Purpose:     Implementation file for the test_SearchProcess_C project.
 *
 * Created:     30th September 2003
 * Updated:     27th December 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (c) 2003-2005, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              This source code is placed into the public domain 2003
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <recls/recls.h>
#include <recls/alloca.h>
#include <recls/assert.h>

#ifdef RECLS_PLATFORM_IS_UNIX
# include <unistd.h>
#endif /* RECLS_PLATFORM_IS_UNIX */

#ifdef RECLS_COMPILER_IS_BORLAND
# include <malloc.h>
#endif /* RECLS_COMPILER_IS_BORLAND */

#if RECLS_VER < RECLS_VER_1_1_1
# error This file now requires version 1.1.1 or later of the recls C API
#endif /* RECLS_VER < 1.1.1 */

#ifdef _MSC_VER
# include <crtdbg.h>
#endif /* _MSC_VER */

/* /////////////////////////////////////////////////////////////////////////////
 * Macros
 */

#ifndef NUM_ELEMENTS
# ifdef __DMC__
#  define NUM_ELEMENTS(x)       (sizeof(x) / sizeof((x)[0]))
# else /* ? compiler */
#  define NUM_ELEMENTS(x)       (sizeof(x) / sizeof(0[(x)]))
# endif /* compiler */
#endif /* !NUM_ELEMENTS */

/* /////////////////////////////////////////////////////////////////////////////
 * Constants and definitions
 */

static size_t path_max(void)
{
#if defined(WIN32)
	return 1 + _MAX_PATH;
#elif defined(PATH_MAX)
	return 1 + PATH_MAX;
#else /* ? PATH_MAX */
	return 1 + 1 + pathconf("/", _PC_PATH_MAX);
#endif /* PATH_MAX */
}

#define alloca_path()	((recls_char_t*)recls_alloca(1 + path_max()))

/* /////////////////////////////////////////////////////////////////////////////
 * Global variables - ouch!
 */

static int      bSuccinct   =   0;
static int      totalFound  =   0;

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

static void usage(int bExit, char const *reason);
void processDirectory(char const *rootDir, char const *pattern, long flags);
int RECLS_CALLCONV_DEFAULT EntryFunc(recls_info_t info, recls_process_fn_param_t param);

/* /////////////////////////////////////////////////////////////////////////////
 * Main
 */

/** Program main */
int main(int argc, char *argv[])
{
#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemState    memState;
#endif /* _MSC_VER && _MSC_VER */
    int             iRet            =   0;
    int             i;
    int             bAllHardDrives  =   0;
    char const      *host	        =   NULL;
    char const      *username       =   NULL;
    char const      *password       =   NULL;
    char const      *pattern        =   NULL;
    char const      *rootDir        =   NULL;
    recls_uint32_t  flags           =   RECLS_F_RECURSIVE;

    for(i = 1; i < argc; ++i)
    {
        const char  *arg    =   argv[i];

        if(arg[0] == '-')
        {
            if(arg[1] == '-')
            {
                /* -- arguments */
            }
            else
            {
                /* - arguments */
                switch(arg[1])
                {
                    case    '?':
                        usage(1, NULL);
                        break;
                    case    'R':    /* Do not recurse */
                        flags &= ~(RECLS_F_RECURSIVE);
                        break;
                    case    'p':    /* Show directory parts */
                        flags |= RECLS_F_DIRECTORY_PARTS;
                        break;
                    case    'f':    /* Find files */
                        flags |= RECLS_F_FILES;
                        break;
                    case    'd':    /* Find directories */
                        flags |= RECLS_F_DIRECTORIES;
                        break;
                    case    'h':    /* Searches from hard drives */
                        bAllHardDrives = 1;
                        break;
                    case    's':    /* Show only the full path; WHEREIS functionality */
                        bSuccinct = 1;
                        break;
                    case    'H':    /* FTP host */
                        host = arg + 2;
                        break;
                    case    'U':    /* FTP username */
                        username = arg + 2;
                        break;
                    case    'P':    /* FTP password */
                        password = arg + 2;
                        break;
#if 0
                    case    'r':    /* Progress */
                        flags |= RECLS_F_DIR_PROGRESS;
                        break;
#endif /* 0 */
                    default:
                        usage(1, "Invalid argument(s) specified");
                        break;
                }
            }
        }
        else
        {
            /* other arguments */
            if(NULL == pattern)
            {
                pattern = arg;
            }
            else if(NULL == rootDir)
            {
                rootDir = arg;
            }
            else
            {
                usage(1, "Invalid argument(s) specified");
            }
        }
    }

	if( NULL != password &&
		(	NULL == host ||
			NULL == username))
	{
		usage(1, "Must specify host and username if specifying password");
	}

	if( NULL != username &&
		NULL == host)
	{
		usage(1, "Must specify host if specifying username");
	}

	if( NULL != host &&
		bAllHardDrives)
	{
		usage(1, "-h flag meaningless to FTP searches");
	}

    if( NULL != host &&
		NULL == rootDir)
    {
		usage(1, "Must specify a root directory for FTP searches");
    }

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemCheckpoint(&memState);
#endif /* _MSC_VER && _MSC_VER */

    /* Search for files if neither files or directories specified/
     *
     * Even though this is not necessary, because the recls API provides the
     * same interpretation, it's best to be explicit.
     */
    if(0 == (flags & (RECLS_F_FILES | RECLS_F_DIRECTORIES)))
    {
        flags |= RECLS_F_FILES;
    }

    if(NULL == pattern)
    {
        pattern = Recls_GetWildcardsAll();
    }

    if(NULL == rootDir)
    {
        rootDir = ".";
    }

    if(bAllHardDrives)
    {
        recls_root_t    roots[26];
        size_t          cRoots  =   Recls_GetRoots(roots, NUM_ELEMENTS(roots));
        size_t          i;

        for(i = 0; i < cRoots; ++i)
        {
            processDirectory(roots[i].name, pattern, flags);
        }
    }
    else
    {
        processDirectory(rootDir, pattern, flags);
    }

    fprintf(stdout, "Total found: %d\n", totalFound);

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemDumpAllObjectsSince(&memState);
#endif /* _MSC_VER && _MSC_VER */

    return iRet;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Function implementations
 */

/** Displays program usage */
void usage(int bExit, char const *reason)
{
    fprintf(stderr, "recls SearchProcess Test Program: test_SearchProcess_C\n\n");

	if(NULL != reason)
	{
		fprintf(stderr, "  Error: %s\n\n", reason);
	}

    fprintf(stderr, "Usage: test_SearchProcess_C [-d] [-f] [-h] [-p] [-R] [-s] [-H<host>] [-U<username>] [-P<password>] [<pattern>] [<root-dir>]\n");
    fprintf(stderr, "\t-R               -   does not recurse; recursive search is the default\n");
    fprintf(stderr, "\t-p               -   evaluate and display directory parts\n");
    fprintf(stderr, "\t-d               -   search for directories\n");
    fprintf(stderr, "\t-f               -   search for files (this is the default, if -d is not specified)\n");
    fprintf(stderr, "\t-h               -   searches from hard drive roots. (Ignore <root-dir>.)\n");
    fprintf(stderr, "\t-s               -   succinct; shows only the path\n");
	fprintf(stderr, "\t-H<host>         -   performs an FTP search on the given host\n");
	fprintf(stderr, "\t-U<username>     -   username to use to log on to <host>\n");
	fprintf(stderr, "\t-P<password>     -   password to use to log on to <host>\n");
    fprintf(stderr, "\t<pattern>        -   search pattern, e.g. \"*.cpp\" (default is to search for all files)\n");
    fprintf(stderr, "\t<root-dir>       -   root directory of search; default is current working directory\n");

    if(bExit)
    {
        exit(EXIT_FAILURE);
    }
}

void processDirectory(char const *rootDir, char const *pattern, long flags)
{
    /* Now just run the search with the given conditions. */
    recls_rc_t  rc = Recls_SearchProcess(rootDir, pattern, flags, EntryFunc, (recls_process_fn_param_t)stdout);

    if(RECLS_FAILED(rc))
    {
        recls_char_t    err[100];

        Recls_GetErrorString(rc, err, 100);

        fprintf(stderr, "Failed to start search, with pattern \"%s\"; recls error: %s\n", pattern, err);
    }
}

/** The callback entry processor function. */
int RECLS_CALLCONV_DEFAULT EntryFunc(recls_info_t info, recls_process_fn_param_t param)
{
    FILE            *stream = (FILE*)param;
    int             i;
    int             off;
    int             extLen;
    int             cDirParts;
    recls_char_t    *path	=	alloca_path();
#ifdef RECLS_PLATFORM_API_WIN32
    recls_char_t    drive;
#endif /* RECLS_PLATFORM_API_WIN32 */
    recls_char_t    *dir		=	alloca_path();
    recls_char_t    *dirPath	=	alloca_path();
    recls_char_t    *file		=	alloca_path();
    recls_char_t    *fileName	=	alloca_path();
    recls_char_t    *fileExt	=	alloca_path();

    recls_char_t    *pathCheck	=	alloca_path();
    recls_char_t    *dirCheck	=	alloca_path();
    recls_char_t    *fileCheck	=	alloca_path();

    ++totalFound;

    pathCheck[0] = '\0';
    dirCheck[0] = '\0';
    fileCheck[0] = '\0';

    Recls_GetPathProperty(info, path, path_max());
    fprintf(stream, "  %s\n", path);
    if(!bSuccinct)
    {
        Recls_GetDirectoryPathProperty(info, dirPath, path_max());
        fprintf(stream, "  %s\n", dirPath);
#ifdef RECLS_PLATFORM_API_WIN32
        Recls_GetDriveProperty(info, &drive);
        if(isupper(path[0]))
        {
            drive = (recls_char_t)toupper(drive);
        }
        else
        {
            drive = (recls_char_t)tolower(drive);
        }
        fprintf(stream, "  %c\n", drive);
        sprintf(pathCheck, "%c:", drive);
#endif /* RECLS_PLATFORM_API_WIN32 */
        Recls_GetDirectoryProperty(info, dir, path_max());
        fprintf(stream, "    %s\n", dir);

        off = 0;
        for(i = 0, cDirParts = Recls_GetDirectoryPartProperty(info, -1, NULL, 0); i < cDirParts; ++i)
        {
            recls_char_t    *dirPart	=	alloca_path();

            off += Recls_GetDirectoryPartProperty(info, i, dirPart, path_max());
            fprintf(stream, "    %*s\n", off, dirPart);

            strcat(pathCheck, dirPart);
            strcat(dirCheck, dirPart);
        }
        if(0 == cDirParts)
        {
            off += strlen(dir);
            strcat(pathCheck, dir);
        }

        off +=   Recls_GetFileProperty(info, file, path_max());
                 Recls_GetFileNameProperty(info, fileName, path_max());
        extLen = Recls_GetFileExtProperty(info, fileExt, path_max());
        fprintf(stream, "    %*s\n", off, file);
        fprintf(stream, "    %*s\n", off - (1 + extLen), fileName);
        fprintf(stream, "    %*s\n", off, fileExt);
        strcpy(fileCheck, fileName);
        if(0 < extLen)
        {
            strcat(fileCheck, ".");
            strcat(fileCheck, fileExt);
        }

        strcat(pathCheck, file);

    /* Now validate the components */
#if defined(RECLS_PLATFORM_IS_WIN32) && \
    !defined(RECLS_PLATFORM_API_WIN32)
            RECLS_ASSERT(0 == strcmp(path + 2, pathCheck));
            if(0 != strcmp(path + 2, pathCheck))
#else /* ? RECLS_PLATFORM_API_WIN32 */
            RECLS_ASSERT(0 == strcmp(path, pathCheck));
            if(0 != strcmp(path, pathCheck))
#endif /* RECLS_PLATFORM_API_WIN32 || !RECLS_PLATFORM_IS_WIN32 */
        {
            fprintf(stderr, "Path is different from path components\n\tpath:  %s\n\tparts: %s\n\n", path, pathCheck);

            abort();
        }
#if defined(RECLS_PLATFORM_API_WIN32)
        RECLS_ASSERT(dirPath[0] == drive && 0 == strcmp(dir, dirPath + 2));
        if( dirPath[0] != drive ||
            0 != strcmp(dir, dirPath + 2))
#elif defined(RECLS_PLATFORM_IS_WIN32)
        RECLS_ASSERT(0 == strcmp(dir, dirPath + 2));
        if(0 != strcmp(dir, dirPath + 2))
#else
        RECLS_ASSERT(dirPath == dir);
        if(dirPath != dir)
#endif /* RECLS_PLATFORM_API_WIN32 || !RECLS_PLATFORM_IS_WIN32 */
        {
            fprintf(stderr, "DirectoryPath is different from Directory\n\tdirPath:  %s\n\tdir: %s\n\n", dirPath, dir);

            abort();
        }
        if(0 != cDirParts)
        {
            RECLS_ASSERT(0 == strcmp(dir, dirCheck));
            if(0 != strcmp(dir, dirCheck))
            {
                fprintf(stderr, "Directory is different from directory components\n\tpath:  %s\n\tparts: %s\n\n", dir, dirCheck);

                abort();
            }
        }
        RECLS_ASSERT(0 == strcmp(file, fileCheck));
        if(0 != strcmp(file, fileCheck))
        {
            fprintf(stderr, "File is different from file components\n\tpath:  %s\n\tparts: %s\n\n", file, fileCheck);

            abort();
        }

        if(Recls_IsFileReadOnly(info))
        {
            fprintf(stream, "    - Read-only\n");
        }
        if(Recls_IsFileDirectory(info))
        {
            fprintf(stream, "    - Directory\n");
        }
        else
        {
            fprintf(stream, "    - File\n");
        }
        if(Recls_IsFileLink(info))
        {
            fprintf(stream, "    - Link\n");
        }
    }

    return 1;
}

/* //////////////////////////////////////////////////////////////////////////
 * End of file
 */
